@@ -1,7 +1,7 @@
 import pytest
 
-from sdmx.message import StructureMessage
-from sdmx.model import Agency, Annotation, Code, Codelist
+from sdmx import message
+from sdmx.model.v21 import Agency, Annotation, Code, Codelist
 
 CL_ITEMS = [
     dict(id="A", name={"en": "Average of observations through period"}),
@@ -39,10 +39,30 @@ def codelist():
     return cl
 
 
-@pytest.fixture
-def structuremessage(codelist):
-    """A StructureMessage for writer testing."""
-    sm = StructureMessage()
-    sm.codelist[codelist.id] = codelist
+# Classes from sdmx.message
 
-    return sm
+
+@pytest.fixture
+def footer():
+    """A :class:`.Footer` for writer testing."""
+    return message.Footer(
+        code=123,
+        severity="information",
+        text=["Here is one string.", "Here is another."],
+    )
+
+
+@pytest.fixture
+def structuremessage(codelist, footer):
+    """A :class:`.StructureMessage.` for writer testing."""
+    return message.StructureMessage(codelist={codelist.id: codelist}, footer=footer)
+
+
+@pytest.fixture
+def datamessage(footer):
+    return message.DataMessage(footer=footer)
+
+
+@pytest.fixture
+def errormessage(footer):
+    return message.ErrorMessage(footer=footer)
