@@ -1,11 +1,13 @@
 from itertools import chain
-from typing import Set, Union
+from typing import Any, Dict, Hashable, Set, Union, cast
 
 import numpy as np
 import pandas as pd
+from pandas.core.indexes.datetimes import prefix_mapping  # type: ignore [attr-defined]
 
-from sdmx import message, model
-from sdmx.model import (
+from sdmx import message
+from sdmx.model import v21 as model
+from sdmx.model.v21 import (
     DEFAULT_LOCALE,
     AllDimensions,
     DataAttribute,
@@ -20,8 +22,8 @@ from sdmx.model import (
 from sdmx.util import DictLike
 from sdmx.writer.base import BaseWriter
 
-#: Default return type for :func:`write_dataset` and similar methods. Either
-#: 'compat' or 'rows'. See the ref:`HOWTO <howto-rtype>`.
+#: Default return type for :func:`write_dataset` and similar methods. Either 'compat' or
+#: 'rows'. See the ref:`HOWTO <howto-rtype>`.
 DEFAULT_RTYPE = "rows"
 
 
@@ -41,7 +43,7 @@ def to_pandas(obj, *args, **kwargs):
 def _list(obj: list, *args, **kwargs):
     """Convert a :class:`list` of SDMX objects."""
     if isinstance(obj[0], Observation):
-        return write_dataset(obj, *args, **kwargs)
+        return write_dataset(model.DataSet(obs=obj), *args, **kwargs)
     elif isinstance(obj[0], DataSet) and len(obj) == 1:
         return writer.recurse(obj[0], *args, **kwargs)
     elif isinstance(obj[0], SeriesKey):
@@ -66,8 +68,8 @@ def _dict(obj: dict, *args, **kwargs):
             # Can safely concatenate these to a pd.MultiIndex'd Series.
             return pd.concat(result)
         else:
-            # The individual pd.Series are indexed by different dimensions; do
-            # not concatenate.
+            # The individual pd.Series are indexed by different dimensions; do not
+            # concatenate
             return DictLike(result)
     elif result_type == {str}:
         return pd.Series(result)
@@ -130,8 +132,8 @@ def write_structuremessage(obj: message.StructureMessage, include=None, **kwargs
     ----------
     obj : .StructureMessage
     include : iterable of str or str, optional
-        One or more of the attributes of the StructureMessage (
-        'category_scheme', 'codelist', etc.) to transform.
+        One or more of the attributes of the StructureMessage ('category_scheme',
+        'codelist', etc.) to transform.
     kwargs :
         Passed to :meth:`write` for each attribute.
 
@@ -194,7 +196,9 @@ def _cr(obj: model.CubeRegion, **kwargs):
     result: DictLike[str, pd.Series] = DictLike()
     for dim, memberselection in obj.member.items():
         result[dim.id] = pd.Series(
-            [mv.value for mv in memberselection.values], name=dim.id
+            # cast(): as of PR#30, only MemberValue is supported here
+            [cast(model.MemberValue, mv).value for mv in memberselection.values],
+            name=dim.id,
         )
     return result
 
@@ -210,67 +214,65 @@ def write_dataset(
 ):
     """Convert :class:`~.DataSet`.
 
-    See the :ref:`walkthrough <datetime>` for examples of using the `datetime`
-    argument.
+    See the :ref:`walkthrough <datetime>` for examples of using the `datetime` argument.
 
     Parameters
     ----------
     obj : :class:`~.DataSet` or iterable of :class:`~.Observation`
     attributes : str
-        Types of attributes to return with the data. A string containing
-        zero or more of:
+        Types of attributes to return with the data. A string containing zero or more
+        of:
 
         - ``'o'``: attributes attached to each :class:`~.Observation` .
-        - ``'s'``: attributes attached to any (0 or 1) :class:`~.SeriesKey`
-          associated with each Observation.
-        - ``'g'``: attributes attached to any (0 or more) :class:`~.GroupKey`
-          associated with each Observation.
+        - ``'s'``: attributes attached to any (0 or 1) :class:`~.SeriesKey` associated
+          with each Observation.
+        - ``'g'``: attributes attached to any (0 or more) :class:`~.GroupKey` associated
+          with each Observation.
         - ``'d'``: attributes attached to the :class:`~.DataSet` containing the
           Observations.
 
     dtype : str or :class:`numpy.dtype` or None
-        Datatype for values. If None, do not return the values of a series.
-        In this case, `attributes` must not be an empty string so that some
-        attribute is returned.
+        Datatype for values. If None, do not return the values of a series. In this
+        case, `attributes` must not be an empty string so that some attribute is
+        returned.
     constraint : .ContentConstraint, optional
         If given, only Observations included by the *constraint* are returned.
     datetime : bool or str or or .Dimension or dict, optional
-        If given, return a DataFrame with a :class:`~pandas.DatetimeIndex`
-        or :class:`~pandas.PeriodIndex` as the index and all other dimensions
-        as columns. Valid `datetime` values include:
+        If given, return a DataFrame with a :class:`~pandas.DatetimeIndex` or
+        :class:`~pandas.PeriodIndex` as the index and all other dimensions as columns.
+        Valid `datetime` values include:
 
-        - :class:`bool`: if :obj:`True`, determine the time dimension
-          automatically by detecting a :class:`~.TimeDimension`.
+        - :class:`bool`: if :obj:`True`, determine the time dimension automatically by
+          detecting a :class:`~.TimeDimension`.
         - :class:`str`: ID of the time dimension.
         - :class:`~.Dimension`: the matching Dimension is the time dimension.
         - :class:`dict`: advanced behaviour. Keys may include:
 
-          - **dim** (:class:`~.Dimension` or :class:`str`): the time dimension
-            or its ID.
-          - **axis** (`{0 or 'index', 1 or 'columns'}`): axis on which to place
-            the time dimension (default: 0).
-          - **freq** (:obj:`True` or :class:`str` or :class:`~.Dimension`):
-            produce :class:`pandas.PeriodIndex`. If :class:`str`, the ID of a
-            Dimension containing a frequency specification. If a Dimension, the
-            specified dimension is used for the frequency specification.
+          - **dim** (:class:`~.Dimension` or :class:`str`): the time dimension or its
+            ID.
+          - **axis** (`{0 or 'index', 1 or 'columns'}`): axis on which to place the time
+            dimension (default: 0).
+          - **freq** (:obj:`True` or :class:`str` or :class:`~.Dimension`): produce
+            :class:`pandas.PeriodIndex`. If :class:`str`, the ID of a Dimension
+            containing a frequency specification. If a Dimension, the specified
+            dimension is used for the frequency specification.
 
-            Any Dimension used for the frequency specification is does not
-            appear in the returned DataFrame.
+            Any Dimension used for the frequency specification is does not appear in the
+            returned DataFrame.
 
     Returns
     -------
     :class:`pandas.DataFrame`
-        - if `attributes` is not ``''``, a data frame with one row per
-          Observation, ``value`` as the first column, and additional columns
-          for each attribute;
+        - if `attributes` is not ``''``, a data frame with one row per Observation,
+          ``value`` as the first column, and additional columns for each attribute;
         - if `datetime` is given, various layouts as described above; or
-        - if `_rtype` (passed from :func:`write_datamessage`) is 'compat',
-          various layouts as described in the :ref:`HOWTO <howto-rtype>`.
+        - if `_rtype` (passed from :func:`write_datamessage`) is 'compat', various
+          layouts as described in the :ref:`HOWTO <howto-rtype>`.
     :class:`pandas.Series` with :class:`pandas.MultiIndex`
         Otherwise.
     """
-    # If called directly on a DataSet (rather than a parent DataMessage),
-    # cannot determine the "dimension at observation level"
+    # If called directly on a DataSet (rather than a parent DataMessage), cannot
+    # determine the "dimension at observation level"
     rtype = kwargs.setdefault("_rtype", "rows")
 
     # Validate attributes argument
@@ -287,8 +289,8 @@ def write_dataset(
         raise ValueError(f"attributes must be in 'osgd'; got {attributes}")
 
     # Iterate on observations
-    data = {}
-    for observation in getattr(obj, "obs", obj):
+    data: Dict[Hashable, Dict[str, Any]] = {}
+    for observation in obj.obs:
         # Check that the Observation is within the constraint, if any
         key = observation.key.order()
         if constraint and key not in constraint:
@@ -299,12 +301,17 @@ def write_dataset(
         if dtype:
             row["value"] = observation.value
         if attributes:
+            # Add the combined attributes from observation, series- and group keys
             row.update(observation.attrib)
+        if "d" in attributes:
+            # Add the attributes of the data set
+            row.update(obj.attrib)
 
         data[tuple(map(str, key.get_values()))] = row
 
+    # NB mypy errors here on CI, but not locally
     result: Union[pd.Series, pd.DataFrame] = pd.DataFrame.from_dict(
-        data, orient="index"
+        data, orient="index"  # type: ignore [arg-type]
     )
 
     if len(result):
@@ -337,11 +344,11 @@ def _dataset_compat(df, datetime, kwargs):
     if obs_dim in (AllDimensions, None):
         pass  # Do nothing
     elif isinstance(obs_dim, TimeDimension):
-        # Don't modify *df*; only change arguments so that
-        # _maybe_convert_datetime performs the desired changes
+        # Don't modify *df*; only change arguments so that _maybe_convert_datetime
+        # performs the desired changes
         if datetime is False or datetime is True:
-            # Either datetime is not given, or True without specifying a
-            # dimension; overwrite
+            # Either datetime is not given, or True without specifying a dimension;
+            # overwrite
             datetime = obs_dim
         elif isinstance(datetime, dict):
             # Dict argument; ensure the 'dim' key is the same as obs_dim
@@ -376,6 +383,8 @@ def _maybe_convert_datetime(df, arg, obj, dsd=None):
         From the `obj` argument to :meth:`write_dataset`.
     dsd: ~.DataStructureDefinition, optional
     """
+    # TODO simplify this method. It has McCabe complexity of 27; the maximum in the rest
+    #      of the code is 17.
     if not arg:
         # False, None, empty dict: no datetime conversion
         return df
@@ -427,12 +436,12 @@ def _maybe_convert_datetime(df, arg, obj, dsd=None):
     # Unstack all but the time dimension and convert
     other_dims = list(filter(lambda d: d != param["dim"], df.index.names))
     df = df.unstack(other_dims)
-    df.index = pd.to_datetime(df.index)
+    df.index = pd.to_datetime(df.index, format="mixed")
 
     if param["freq"]:
         # Determine frequency string, Dimension, or Attribute
         freq = param["freq"]
-        if isinstance(freq, str) and freq not in pd.offsets.prefix_mapping:
+        if isinstance(freq, str) and freq not in prefix_mapping:
             # ID of a Dimension or Attribute
             for component in chain(_get_dims(), _get_attrs()):
                 if component.id == freq:
@@ -492,7 +501,7 @@ def write_itemscheme(obj: model.ItemScheme, locale=DEFAULT_LOCALE):
 
     Returns
     -------
-    pandas.Series
+    pandas.Series or pandas.DataFrame
     """
     items = {}
     seen: Set[Item] = set()
@@ -505,15 +514,12 @@ def write_itemscheme(obj: model.ItemScheme, locale=DEFAULT_LOCALE):
         else:
             seen.add(item)
 
-        # Localized name
-        row = {"name": item.name.localized_default(locale)}
-        try:
+        items[item.id] = dict(
+            # Localized name
+            name=item.name.localized_default(locale),
             # Parent ID
-            row["parent"] = item.parent.id
-        except AttributeError:
-            row["parent"] = ""
-
-        items[item.id] = row
+            parent=item.parent.id if isinstance(item.parent, item.__class__) else "",
+        )
 
         # Add this item's children, recursively
         for child in item.child:
@@ -523,9 +529,11 @@ def write_itemscheme(obj: model.ItemScheme, locale=DEFAULT_LOCALE):
         add_item(item)
 
     # Convert to DataFrame
-    result = pd.DataFrame.from_dict(items, orient="index", dtype=object).rename_axis(
-        obj.id, axis="index"
-    )
+    result: Union[pd.DataFrame, pd.Series] = pd.DataFrame.from_dict(
+        items,
+        orient="index",
+        dtype=object,  # type: ignore [arg-type]
+    ).rename_axis(obj.id, axis="index")
 
     if len(result) and not result["parent"].str.len().any():
         # 'parent' column is empty; convert to pd.Series and rename
